﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Linq;
using BlazorAppEF.Data;
using Microsoft.EntityFrameworkCore;

namespace BlazorAppEF.Services
{
    public interface ICustomersService
    {
        Task<Customer> GetCustomerById(int id);
        Task<IReadOnlyList<Customer>> GetAllCustomersFromDb();
        Task<List<Customer>> SearchCustomers(CustomerFilter customerFilter);
        Task<bool> AddCustomer(Customer customer);
        Task<bool> UpdateCustomer(Customer customer);
        Task<bool> DeleteCustomer(Customer customer);
    }

    public class CustomersService : ICustomersService
    {
        private readonly TestDbContext _dbContext;
        public CustomersService(TestDbContext dbContext)
        {
            _dbContext = dbContext;
        }

        public async Task<Customer> GetCustomerById(int customerId)
        {
            Customer customer = _dbContext.Customers.Single(x => x.CustomerId == customerId);
            return await Task.Factory.StartNew(() => customer);
        }

        public async Task<IReadOnlyList<Customer>> GetAllCustomersFromDb()
        {
            try
            {
                IReadOnlyList<Customer> customers = await _dbContext.Set<Customer>().ToListAsync();

                return customers;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public async Task<List<Customer>> SearchCustomers(CustomerFilter customerFilter)
        {
            try
            {
                IEnumerable<Customer> customers = _dbContext.Customers as IEnumerable<Customer>;

                if (customerFilter.CustomerId != 0)
                    customers = _dbContext.Customers.Where(x => x.CustomerId == customerFilter.CustomerId);

                if (!string.IsNullOrEmpty(customerFilter.CustomerName))
                    customers = customers.Where(x => x.CustomerName.Contains(customerFilter.CustomerName));

                if (!string.IsNullOrEmpty(customerFilter.Address))
                    customers = customers.Where(x => x.Address.Contains(customerFilter.Address));

                if (!string.IsNullOrEmpty(customerFilter.Telephone))
                    customers = customers.Where(x => x.Telephone.Contains(customerFilter.Telephone));

                if (!string.IsNullOrEmpty(customerFilter.Website))
                    customers = customers.Where(x => x.Website.Contains(customerFilter.Website));

                if (!string.IsNullOrEmpty(customerFilter.Email))
                    customers = customers.Where(x => x.Email.Contains(customerFilter.Email));

                if(customerFilter.PageIndex > -1)
                    customers = customers.Skip(customerFilter.PageIndex* customerFilter.PageSize);

                if (customerFilter.PageSize != 0)
                    customers = customers.Take(customerFilter.PageSize);

                List<Customer> data = customers.ToList();
                return await Task.Factory.StartNew(() => data);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public async Task<bool> AddCustomer(Customer customer)
        {
            try
            {
                _dbContext.Customers.Add(customer);
                await _dbContext.SaveChangesAsync();
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        public async Task<bool> UpdateCustomer(Customer customer)
        {
            try
            {
                _dbContext.Entry(customer).State = EntityState.Modified;
                await _dbContext.SaveChangesAsync();

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        public async Task<bool> DeleteCustomer(Customer customer)
        {
            try
            {
                _dbContext.Customers.Remove(customer);
                await _dbContext.SaveChangesAsync();

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }
    }
}
